import cfn_sweeper.resources.AWS_S3_Bucket as bucket
import pytest
import boto3
from moto import mock_s3

@mock_s3
def test_s3_resource():
    s3_scanner_resource = bucket.resource()
    
    scan_result = s3_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0
    
    client = boto3.client('s3', region_name='us-east-1')
    client.create_bucket(Bucket='bucket1')
    client.create_bucket(Bucket='bucket2')
  
    assert s3_scanner_resource.resource_name == 'AWS::S3::Bucket'
    
    scan_result = s3_scanner_resource.gather(region='us-east-1')
    assert len(scan_result) == 2
    assert 'bucket1' in scan_result
    assert 'bucket2' in scan_result

    client.delete_bucket(Bucket='bucket1')
    client.delete_bucket(Bucket='bucket2')
    
    # test pagination
    for num in range(500):
        client.create_bucket(Bucket='bucket{}'.format(
            num
        ))
    scan_result = s3_scanner_resource.gather(region='us-east-1')
    assert len(scan_result) == 500

@mock_s3
def test_s3_resource_multi_region():

    s3_scanner_resource = bucket.resource()
    
    scan_result = s3_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0

    client = boto3.client('s3', region_name='us-east-1')

    east_bucket_name='us-east-1-bucket'
    west_bucket_name='us-west-1-bucket'
    client.create_bucket(Bucket=east_bucket_name)
    client.create_bucket(Bucket=west_bucket_name, CreateBucketConfiguration={'LocationConstraint':'us-west-1'})

    us_east_1_scan_result = s3_scanner_resource.gather(region='us-east-1')
    us_west_1_scan_result = s3_scanner_resource.gather(region='us-west-1')

    assert len(us_east_1_scan_result) == 1
    assert len(us_west_1_scan_result) == 1
    assert east_bucket_name not in us_west_1_scan_result
    assert west_bucket_name not in us_east_1_scan_result


import sys
import re
import argparse

class ValidateRegion(argparse.Action):
    """Validate Region"""
    def __call__(self, parser, namespace, values, option_string=None):
        regex = re.compile('(us(-gov)?|ap|ca|cn|eu|sa)-(central|(north|south)?(east|west)?)-\d', re.I)
        match = regex.match(str(values))
        if bool(match) == False:
            sys.exit("--region parameter must be listed in the code portion of this table: https://docs.aws.amazon.com/general/latest/gr/rande.html#regional-endpoints")
        else:
            setattr(namespace, self.dest, values)

class Validateoutput(argparse.Action):
    """Validate Output"""        
    def __call__(self, parser, namespace, values, option_string=None):
        output = ['pretty', 'json', 'yaml','stdout']
        if not values in output:
            sys.exit("--output parameter currently only supports: " + str(output))
        else:
            setattr(namespace, self.dest, values)

class Validatefilter(argparse.Action):
    """Validate Filter Types"""
    def __call__(self, parser, namespace, values, option_string=None):
        regex = re.compile('AWS::[0-z]*::[0-z]*', re.I)
        for value in values:
           match = regex.match(str(value))
        if match:
            setattr(namespace, self.dest, values)
        else:
            sys.exit("--filter-types parameter not valid, please look at the README and check: https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-template-resource-type-ref.html ")



from importlib import import_module
from pkgutil import walk_packages
import os

def get_aws_modules(files:list) -> list:
    '''
    Given a list of files, returns a list of the files that are
    prefixed with the AWS_ prefix

    This is used to filter out files that are required in the directory,
    such as __init__py or the abstract class 

    Parameters:
        files (list): an Array of strings - file names
    '''

    return list(filter(lambda x: x.startswith('AWS_'), files))

def get_module_dir() -> str:
    '''
    Returns the absolute directory of the resources folder - where the modules
    for AWS resources are located
    '''
    root_dir = os.path.dirname(os.path.abspath(__file__))
    root_dir_len = len(root_dir)
    return root_dir[:root_dir_len - 4] + 'resources'

def get_package_modules(package_path:str) -> list:
    result = []
    for package in walk_packages([package_path]):
        result.append(package.name)

    return result


class PluginManager:
    '''
    This class is used to manage and interact with the various AWS resource modules
    stored in the cfn_sweeper/resources section of the project. Upon initialization -
    the PluginManager class will dynamically load each module, and load it into a
    dict object with a key of the resource_name (eg - AWS::S3::Bucket or 
    AWS::EC2::Instance), and the modules corresponding gather() function as the value. 
    The dict shouldn't be directly interacted with - instead this class exposes a 
    gather_resource() method that expects a cloudformation resource_name, and aws region.
    The method will then look for a matching key in the dict - and execute its associated
    gather() method with the provided region parameter.
    '''
    
    def __init__(self):
        self.modules = {}
        module_dir_files = get_package_modules(get_module_dir())
        aws_modules = get_aws_modules(module_dir_files)
        for module in aws_modules:
            
            module_path = 'cfn_sweeper.resources.{}'.format(module.split('.')[0])
            try:
                loaded_module = import_module(module_path)
                resource = loaded_module.resource()
                self.modules[resource.resource_name] = resource.gather

            except ModuleNotFoundError as e:
                print(e)
                print('unable to load {}'.format(module))

    def gather_resource(self, region:str, resource_name:str):
        try:
            result = self.modules[resource_name](region=region)
            return result
        except KeyError as e:
            print(e)
            raise NotImplementedError


from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):
    
    def __init__(self):
        self.resource_name = "AWS::EC2::Instance"

#Can return the below as well if needed: ,    "Platform:": instance.platform,   "Type:": instance.instance_type,   "PublicIPv4:": instance.public_ip_address, "AMI:": #instance.image.id, "State:": instance.state
    def gather(self, region):
        result = []
        ec2 = boto3.resource('ec2',region_name=region)
        for instance in ec2.instances.all():
            result.append(instance.id)
        return result



from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):
    
    def __init__(self):
        self.resource_name = "AWS::EFS::FileSystem"

    def gather(self, region: str):
        results = []
        efs = boto3.client('efs',  region_name=region)
        fs_paginator = efs.get_paginator('describe_file_systems')
        for response in fs_paginator.paginate():
            response_fs_names = [r.get('FileSystemId') for r in response['FileSystems']]
            results.extend(response_fs_names)
        return results


from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):

    def __init__(self):
        self.resource_name = "AWS::Lambda::Function"

    def gather(self, region):
        results = []
        client = boto3.client('lambda', region_name=region)
        function_paginator = client.get_paginator('list_functions')
        for response in function_paginator.paginate():
            response_function_names = [r.get('FunctionName') for r in response['Functions']]
            results.extend(response_function_names)
        return results


# test_AWS_DynamoDB_Table.py
import cfn_sweeper.resources.AWS_DynamoDB_Table as dynamo
import pytest
import boto3 # AWS SDK for Python
from moto import mock_dynamodb2 # since we're going to mock DynamoDB service

@mock_dynamodb2
def test_dynamo_db():
    
    dynamo_scanner_resource = dynamo.resource()
    
    
    scan_result = dynamo_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0    
    
    """
    Create database resource and mock table
    """
    dynamodb = boto3.client('dynamodb', region_name='us-east-1')
    dbcnt = 0
    while dbcnt < 105:
        dbcnt += 1
        dynamodb.create_table(TableName="TableName" + str(dbcnt),
                                  KeySchema=[
                                      {
                                            'AttributeName': 'Test',
                                            'KeyType': 'HASH'
                                      }
                                  ],
                                  AttributeDefinitions=[
                                      {
                                          'AttributeName': 'Test',
                                          'AttributeType': 'N'
                                      }
                                  ],
                                  ProvisionedThroughput={
                                      'ReadCapacityUnits': 1,
                                      'WriteCapacityUnits': 1
                                  }
                                  )
        

    """
    Test if our mock table is ready and we can return it
    """
    
    assert dynamo_scanner_resource.resource_name == 'AWS::DynamoDB::Table'
    
    
    scan_result = dynamo_scanner_resource.gather(region='us-east-1')


    assert len(scan_result) == dbcnt







from cfn_sweeper.base.cfn_resources import is_managed_by_cloudformation, get_all_cfn_resources_by_type, load_cfn_resources
import pytest
import boto3
import json
from moto import mock_cloudformation

@mock_cloudformation
def test_load_cfn_resources():
    
    dummy_template1 = {
    "AWSTemplateFormatVersion": "2010-09-09",
    "Description": "Stack 3",
    "Resources": {
        "VPC": {"Properties": {"CidrBlock": "192.168.0.0/16"}, "Type": "AWS::EC2::VPC"}
    },
    }

    dummy_template2 = {
    "AWSTemplateFormatVersion": "2010-09-09",
    "Description": "Stack 1",
    "Resources": {
        "EC2Instance1": {
            "Type": "AWS::EC2::Instance",
            "Properties": {
                "ImageId": '11111111122334324',
                "KeyName": "dummy",
                "InstanceType": "t2.micro",
                "Tags": [
                    {"Key": "Description", "Value": "Test tag"},
                    {"Key": "Name", "Value": "Name tag for tests"},
                ],
            },
        }
    },
}
    
    client = boto3.client('cloudformation', 'us-east-1')
    client.create_stack(StackName="test_stack1", TemplateBody=json.dumps(dummy_template1))
    client.create_stack(StackName="test_stack2", TemplateBody=json.dumps(dummy_template2))

    cfn_resources = load_cfn_resources('us-east-1')

    assert len(cfn_resources) == 2
    assert cfn_resources[0]['PhysicalResourceId'] 
    assert cfn_resources[0]['ResourceType']
    assert cfn_resources[0]['ResourceStatus']
    
    client.delete_stack(StackName="test_stack1")
    client.delete_stack(StackName="test_stack2")

    cfn_resources = load_cfn_resources('us-east-1')
    assert len(cfn_resources) == 0

def test_get_all_cfn_resources_by_type():
    resource_array = [{
        "ResourceType":"AWS::EC2::Instance"
    },{
        "ResourceType":"AWS::Lambda::Function"
    },
    {
        "ResourceType":"AWS::IAM::Role"
    }]

    result = get_all_cfn_resources_by_type(resource_array, 'AWS::EC2::Instance')

    assert type(result) is list
    assert len(result) == 1
    assert result[0]['ResourceType'] == 'AWS::EC2::Instance'

    resource_array = [{
        "ResourceType":"AWS::Lambda::Function"
    },{
        "ResourceType":"AWS::Lambda::Function"
    },
    {
        "ResourceType":"AWS::IAM::Role"
    }]

    result = get_all_cfn_resources_by_type(resource_array, 'AWS::Lambda::Function')

    assert len(result) == 2
    assert result[0]['ResourceType'] == 'AWS::Lambda::Function'
    assert result[1]['ResourceType'] == 'AWS::Lambda::Function'

    result = get_all_cfn_resources_by_type(resource_array, 'AWS::EC2::VPC')

    assert len(result) == 0

def test_is_managed_by_cloudformation():
    resource_array = [{
        "PhysicalResourceId":"vpc-1234"
    },{
        "PhysicalResourceId":"vpc-5678"
    },{
        "PhysicalResourceId":"vpc-7891"
    }]


    assert is_managed_by_cloudformation("vpc-1234", resource_array) == True
    assert is_managed_by_cloudformation("vpc-7891", resource_array) == True
    assert is_managed_by_cloudformation("vpc-1112", resource_array) == False


import argparse
from cfn_sweeper.base.cfn_resources import load_cfn_resources, get_all_cfn_resources_by_type, is_managed_by_cloudformation
from cfn_sweeper.base.runner import PluginManager
from cfn_sweeper.base.output import ScanReport
from cfn_sweeper.validation import ValidateRegion,Validateoutput,Validatefilter

    

def main():

    parser = argparse.ArgumentParser()
    parser.add_argument('--region',
                        help='Enter a region like us-east-2.',
                        dest="region",
                        action=ValidateRegion,
                        required=True)
    parser.add_argument('--output',
                        help='pretty, json, yaml, stdout',
                        dest="output",
                        action=Validateoutput,
                        nargs="?",
                        default="yaml"
                        )
    parser.add_argument('--filter-types',
                        help='eg: AWS::IAM::Role or AWS::EC2::Instance.',
                        nargs='+',
                        dest="types",
                        action=Validatefilter,
                        required=True)
    parser.add_argument('--tag_keys',
                        help='Allows you to exclude particular AWS Resources based on the presence of a particular tag key on the resource. This will only be applied to AWS Resources that support tagging. Valid values: any string that is a valid tag - multiple values can be supplied.',
                        dest="tags")

    #TODO: add argument validation - including regex for patterns
    args = parser.parse_args()

    #TODO: inital shot at this - refactor later
    region = args.region
    types = args.types
    output = args.output
    cfn_resources = load_cfn_resources(region)
    runner = PluginManager()
    
    #TODO: abstract result into its own class - so we can easily output to different formats
    report = ScanReport() 
    for resource_type in types:
        resources_in_cloudformation = get_all_cfn_resources_by_type(cfn_resources, resource_type)
        try:
            resources_in_account = runner.gather_resource(region=region, resource_name=resource_type)
        except NotImplementedError:
            print('Sorry - {} isn''t supported just yet!'.format(
                resource_type
            ))
            continue

        managed_resources = []
        unmanaged_resources = []
        for resource in resources_in_account:
            cfn_managed = is_managed_by_cloudformation(physical_resource_id=resource, resource_array=resources_in_cloudformation)
            if cfn_managed:
              managed_resources.append(resource)
            else:
              unmanaged_resources.append(resource)
        report.add_resource_results(resource_type=resource_type, managed_resources=managed_resources, unmanaged_resources=unmanaged_resources)
    
    
    if output == 'yaml':
      report.print_to_yaml()
    elif output == 'json':
      report.print_to_json()
    elif output == 'stdout':
      report.print_to_stdout()
    elif output == 'pretty':
      report.print_to_pretty()
    










    

    


if __name__ == '__main__':
    main()



import cfn_sweeper.resources.AWS_EC2_Instance as ec2
import pytest
import boto3
from moto import mock_ec2

@mock_ec2
def test_ec2_resource():
    ec2_scanner_resource = ec2.resource()
    
    scan_result = ec2_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0
    
    client = boto3.resource('ec2', region_name='us-east-1')
    client.create_instances(
    ImageId= 'aki-00806369',
    MinCount=1,
    MaxCount=1,
    InstanceType='t2.micro',
)

    
    assert ec2_scanner_resource.resource_name == 'AWS::EC2::Instance'
    
    scan_result = ec2_scanner_resource.gather(region='us-east-1')

    assert len(scan_result) == 1

    
    



from cfn_sweeper import resources
from cfn_sweeper.resources import AWS_Lambda_Function as aws_lambda
import zipfile
import io
import pytest
import boto3
from botocore.exceptions import ClientError
from moto import mock_lambda, mock_iam


@mock_lambda
def test_lambda_resoure():
    lambda_scanner_resource = aws_lambda.resource()

    assert len(lambda_scanner_resource.gather('us-east-1')) == 0

    client = boto3.client('lambda', region_name='us-east-1')
    client.create_function(
        FunctionName="testFunction",
        Role=get_role_name(),
        Code ={"ZipFile": get_test_zip_file1()},
        Handler="lambda_function.lambda_handler",
        Runtime='python2.7'
    )

    assert len(lambda_scanner_resource.gather('us-east-1')) == 1
    assert len(lambda_scanner_resource.gather('us-east-2')) == 0

    for x in range(0,500):
        client.create_function(
            FunctionName='MyTestFunction{}'.format(str(x)),
            Role=get_role_name(),
            Code ={"ZipFile": get_test_zip_file1()},
            Handler="lambda_function.lambda_handler",
            Runtime='python2.7'
        )

    assert len(lambda_scanner_resource.gather('us-east-1')) == 501
    assert len(lambda_scanner_resource.gather('us-east-2')) == 0



def get_role_name():
    with mock_iam():
        iam = boto3.client("iam")
        try:
            return iam.get_role(RoleName="my-role")["Role"]["Arn"]
        except ClientError:
            return iam.create_role(
                RoleName="my-role",
                AssumeRolePolicyDocument="some policy",
                Path="/my-path/",
            )["Role"]["Arn"]


def _process_lambda(func_str):
    zip_output = io.BytesIO()
    zip_file = zipfile.ZipFile(zip_output, "w", zipfile.ZIP_DEFLATED)
    zip_file.writestr("lambda_function.py", func_str)
    zip_file.close()
    zip_output.seek(0)
    return zip_output.read()


def get_test_zip_file1():
    pfunc = """
def lambda_handler(event, context):
    print("custom log event")
    return event
"""
    return _process_lambda(pfunc)



from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):
    
    def __init__(self):
        self.resource_name = "AWS::S3::Bucket"

    def gather(self, region):
        results = []
        s3_client = boto3.client('s3', region_name=None)
        raw_response = s3_client.list_buckets()
        for bucket in raw_response['Buckets']:
            bucket_region = s3_client.get_bucket_location(Bucket=bucket['Name'])['LocationConstraint']
            if region == 'us-east-1':
                if bucket_region == None:
                   results.append(bucket['Name']) 
            elif bucket_region == region:
                results.append(bucket['Name'])
           

        return results



from abc import ABC, abstractmethod

class base_resource(ABC):
    """
    Abstract class for adding individual resource scanning support to cfn_sweeper
    All classes built from this require the resource_name property to be defined,
    and a gather() method to be implemented on the subclass.

    The resource_name property should be the cloudformation type of the resource
    (eg AWS::S3::Bucket).

    The gather() method should take a region property, and return a list of all physical
    resource ids for the resource type in question (for S3 Buckets - given the region of
    us-east-1, it would return a list of all the bucket names in us-east-1)
    """
    def resource_name(self):
        try:
            return self._resource_name
        except AttributeError:
            raise NotImplementedError('base_resources are required to have a resource name set')

    @abstractmethod
    def gather(self, region:str):
        pass


from cfn_sweeper.base.output import ScanReport
import pytest

def test_ScanReport(capsys):

    report = ScanReport()
    report.add_resource_results(resource_type='AWS::S3::Bucket', managed_resources=['mycfnmadebucket'], unmanaged_resources=['mynoncfnbucket'])

    assert report._ScanReport__report_data == {
        'AWS::S3::Bucket':{
            'managed':['mycfnmadebucket'],
            'unmanaged':['mynoncfnbucket']
        }
    }

    report.print_to_json()
    captured = capsys.readouterr()
    assert captured.out == '{"AWS::S3::Bucket": {"managed": ["mycfnmadebucket"], "unmanaged": ["mynoncfnbucket"]}}\n'
    
    report.print_to_yaml()
    captured = capsys.readouterr()
    assert captured.out == 'AWS::S3::Bucket:\n  managed:\n  - mycfnmadebucket\n  unmanaged:\n  - mynoncfnbucket\n\n'

    report.print_to_stdout()
    captured = capsys.readouterr()
    assert captured.out == 'mynoncfnbucket\n'


import boto3
from botocore.config import Config

cfn_config = Config(
    retries={
        'max_attempts': 10,
        'mode': 'adaptive'
    }
)


def load_cfn_resources(region: str) -> list:
    """
     Gets all the Cloudformation managed resources for the given AWS region

     Parameters:
         region (string): the AWS Region to scan

     Returns:
         An array of dict - containing the information of Cloudformation resources

    """
    cloudformation_client = boto3.client(
        service_name='cloudformation', region_name=region, config=cfn_config)
    stacks_in_account = []
    stack_paginator = cloudformation_client.get_paginator('list_stacks')

    stack_page_iterator = stack_paginator.paginate(
        StackStatusFilter=['CREATE_COMPLETE', 'UPDATE_COMPLETE'])
    for page in stack_page_iterator:
        stacks_in_account.extend(page['StackSummaries'])
    resource_paginator = cloudformation_client.get_paginator(
        'list_stack_resources')
    result = []
    for stack in stacks_in_account:
        resource_page_iterator = resource_paginator.paginate(
            StackName=stack['StackName'])
        for page in resource_page_iterator:
            for resource in page['StackResourceSummaries']:
                result.append(resource)
    return result


def get_all_cfn_resources_by_type(resource_array: list, resource_type: str) -> list:
    """
    Given a list of cloudformation stack resources, filters the resources by the specified type

    Parameters:
        resource_array (list): an Array of Cloudformation Stack Resources
        resource_type (string): the Name of the Cloudformation Resource type to filter for - example: AWS::EC2::Instance

    Returns:
        An array of dict - containing the filtered Cloudformation resources
    """
    result = []

    for resource in resource_array:
        if resource['ResourceType'] == resource_type:
            result.append(resource)
    return result


def is_managed_by_cloudformation(physical_resource_id: str, resource_array: list) -> bool:
    """
    Given a physical resource id and array of rources - returns if the resource is managed by cloudformation

    Parameters:
        physical_resource_id (string): The identifier of the resource - eg igw-09a7b4932e331edb2 or vpc-054a63a50efe678b3
        resource_array (list): an array of cloudformation stack resources

    Returns:
        boolean - if the resource is found in the list of resource_array
    """

    for resource in resource_array:
        if resource['PhysicalResourceId'] == physical_resource_id:
            return True
    else:
        return False



import cfn_sweeper.resources.AWS_EFS_FileSystem as efs
import pytest
import boto3
from moto import mock_efs

@mock_efs
def test_efs_resource():

    efs_fs_scanner_resource = efs.resource()

    assert efs_fs_scanner_resource.resource_name == 'AWS::EFS::FileSystem'

    scan_result = efs_fs_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0
    
    client = boto3.client('efs', region_name='us-east-1')
    client.create_file_system()

    assert len(efs_fs_scanner_resource.gather(region='us-east-1')) == 1
    assert len(efs_fs_scanner_resource.gather(region='us-east-2')) == 0

    for x in range(0,100):
        client.create_file_system()

    assert len(efs_fs_scanner_resource.gather(region='us-east-1')) == 101



from re import X
from pyfiglet import Figlet
class Artwork():
    
        def art():
          f = Figlet(font='larry3d')
          print("\n https://github.com/rileydakota/cfn-sweeper" )
          print (f.renderText('CFN SWEEPER \n'))  
          print(("          The umanaged resource detector tool!").center(20))
          print(("-----------------------------------------------------------").center(24, " "))
          print(("                    Run Report").center(20))
          print(("-----------------------------------------------------------").center(24, " "))


from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):

    def __init__(self):
        self.resource_name = "AWS::KMS::Key"

    def gather(self, region):
        results = []
        kms = boto3.client('kms', region_name=region)
        paginator = kms.get_paginator('list_keys')
        for response in paginator.paginate():
            response_key_ids = [r.get('KeyId') for r in response['Keys']]
            results.extend(response_key_ids)
        return results


import cfn_sweeper.resources.AWS_KMS_Key as kms
import pytest
import json
import boto3 # AWS SDK for Python
from moto import mock_kms


@mock_kms
def test_kms():

    kms_scanner_resource = kms.resource()

    scan_result = kms_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0

    kms_client = boto3.client('kms', region_name='us-east-1')
    kmscnt = 0
    while kmscnt < 10:
        kmscnt += 1
        kms_client.create_key()

    scan_result_us1 = kms_scanner_resource.gather('us-east-1')
    assert len(scan_result_us1) == 10


    scan_result_us2 = kms_scanner_resource.gather('us-east-2')
    assert len(scan_result_us2) == 0
    
    kms_client_other_region = boto3.client('kms', region_name='us-east-2')
    kmscnt = 0
    while kmscnt < 10:
        kmscnt += 1
        kms_client_other_region.create_key()
    
    scan_result_us1 = kms_scanner_resource.gather('us-east-1')
    scan_result_us2 = kms_scanner_resource.gather('us-east-2')
    assert len(scan_result_us1) == 10
    assert len(scan_result_us2) == 10 






from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):

    def __init__(self):
        self.resource_name = "AWS::IAM::Role"

    def gather(self, region):
        results = []
        iam = boto3.client('iam', region_name=region)
        role_paginator = iam.get_paginator('list_roles')
        for response in role_paginator.paginate():
            response_role_names = [r.get('RoleName') for r in response['Roles']]
            results.extend(response_role_names)
        return results
        
        



from cfn_sweeper.resources.base import base_resource
import boto3

class resource(base_resource):
    
    def __init__(self):
        self.resource_name = "AWS::DynamoDB::Table"


    def gather(self, region):
        result = []
        dynamodb = boto3.client('dynamodb',region_name=region)
        response = dynamodb.list_tables()
        for table in response['TableNames']:
            result.append(table)

        while 'LastEvaluatedTableName' in response:
            
            response = dynamodb.list_tables(ExclusiveStartTableName = response['LastEvaluatedTableName'])
            for i in response['TableNames']:
                result.append(i)
        else:   
            return result
 


from cfn_sweeper.base.runner import PluginManager, get_aws_modules, get_module_dir, get_package_modules
import uuid
import pytest

def test_get_aws_modules():
    list_dir = [
        'AWS_EC2_Instance.py',
        'AWS_IAM_Role.py',
        'NOT_AN_AWS_MODULE',
        '__init__.py',
        'base.py'
    ]

    result = get_aws_modules(list_dir)
    assert '__init__.py' not in result
    assert 'base.py' not in result
    assert 'NOT_AN_AWS_MODULE' not in result
    assert len(result) == 2
    assert 'AWS_EC2_Instance.py' in result
    assert 'AWS_IAM_Role.py' in result

def test_get_module_dir(mocker):
    mocker.patch(
        'cfn_sweeper.base.runner.os.path.dirname',
        return_value='/home/randomuser/cfn-sweeper/cfn-sweeper/base'
    )
    
    plugin_dir = get_module_dir()
    assert plugin_dir == '/home/randomuser/cfn-sweeper/cfn-sweeper/resources'

def test_get_package_modules(mocker):
    mocker.patch(
        'cfn_sweeper.base.runner.walk_packages',
        return_value=[
            FakePkgUtilModuleClass(name='module1'),
            FakePkgUtilModuleClass(name='module2')    
        ]
    )

    result = get_package_modules('module_path')
    assert len(result) == 2
    assert 'module1' in result
    assert 'module2' in result

def test_PluginManager(mocker):
    mocker.patch(
        'cfn_sweeper.base.runner.get_package_modules',
        return_value=[
            'AWS_EC2_Instance.py',
            'AWS_S3_Bucket.py',
            '__init__.py',
            'NOT_AN_AWS_MODULE.py'
        ]
    )

    mocker.patch(
        'cfn_sweeper.base.runner.os.path.dirname',
        return_value='/home/randomuser/cfn-sweeper/cfn-sweeper/base'
    )
 
    mocker.patch(
        'cfn_sweeper.base.runner.import_module',
        return_value=FakeModuleClass()
    )

    runner = PluginManager()
    assert runner.gather_resource(region='us-east-1', resource_name='AWS::Resource::Thing') == [
        'aws-thing-1',
        'aws-thing-2'
    ]

    with pytest.raises(NotImplementedError) as error_info:
        runner.gather_resource(region='us-east-1', resource_name='AWS::S3::Bucket')
    assert "NotImplementedError" in str(error_info)

""" Sadly not sure of a better way to test this - but essentually creating fake classes to simulate
    how our plugins would behave when dynamically imported using import_lib
"""

class FakeModuleClass():
    def resource(self):
        return FakeResourceClass()

class FakeResourceClass():
    def __init__(self):
        self.resource_name = 'AWS::Resource::Thing'

    def gather(self, region):
        return [
            'aws-thing-1',
            'aws-thing-2'
        ]

class FakePkgUtilModuleClass():
    def __init__(self, name):
        self.name = name


from setuptools import setup, find_packages

VERSION = '0.0.6'
DESCRIPTION = 'A tool for finding resources unmanaged by cloudformation'


setup(
    name="cfn-sweeper",
    version=VERSION,
    py_modules=["main"],
    entry_points={
        'console_scripts': [
            'cfn_sweeper=cfn_sweeper.main:main',
        ],
    },
    description=DESCRIPTION,
    packages=find_packages(),
    install_requires=[
        'boto3',
        'wheel',
        'pyyaml',
        'pyfiglet'
    ],
    python_requires=">=3.8"
    
)



import argparse
from cfn_sweeper.validation import ValidateRegion,Validateoutput,Validatefilter
import pytest 
from contextlib import contextmanager


@contextmanager
def not_raises(exception):
        try:
            yield
        except exception:
            raise pytest.fail("DID RAISE {0}".format(exception))


def test_required_unknown():
    """ Try to perform sweep on something that isn't an option. """
    parser=argparse.ArgumentParser()
    parser.add_argument('--region',
                        help='Enter a region like us-east-2.',
                        dest="region",
                        action=ValidateRegion,
                        required=True)
    parser.add_argument('--output',
                        help='pretty, json, yaml',
                        dest="output",
                        action=Validateoutput,
                        nargs="?",
                        default="yaml"
                        )
    parser.add_argument('--filter-types',
                        help='eg: AWS::IAM::Role or AWS::EC2::Instance. Using  "ALL"  with no quotes and we will run it for all current supported resource types',
                        nargs='+',
                        dest="types",
                        action=Validatefilter,
                        required=True)
    parser.add_argument('--tag_keys',
                        help='Allows you to exclude particular AWS Resources based on the presence of a particular tag key on the resource. This will only be applied to AWS Resources that support tagging. Valid values: any string that is a valid tag - multiple values can be supplied.',
                        dest="tags")
    
    #This should raise an error since this will cause a SystemExit since bad params were passed in 
    args = ["--region", "NADA",'--output', "NADA",'--filter-types',"NADA"]
    with pytest.raises(SystemExit):
        parser.parse_args(args)
        
        
        
        
    #This should NOT raise an error since good params were passed into the parser
    args = ["--region", "us-east-1",'--output', "yaml",'--filter-types',"AWS::EC2::Instance"]    
    with not_raises(SystemExit):
        parser.parse_args(args)


# test_AWS_IAM_Role.py
import cfn_sweeper.resources.AWS_IAM_Role as iam
import pytest
import json
import boto3 # AWS SDK for Python
from moto import mock_iam # since we're going to mock DynamoDB service

@mock_iam
def test_iam():
    
    iam_scanner_resource = iam.resource()
    
    
    scan_result = iam_scanner_resource.gather('us-east-1')
    assert len(scan_result) == 0    
    
    """
    Create and mock role creation and trust relation
    """
    
    assume_role_policy_document = json.dumps({
        "Version": "2012-10-17",
        "Statement": [
            {
                "Effect": "Allow",
                "Principal": {
                    "Service": "support.amazonaws.com"
                },
                "Action": "sts:AssumeRole"
            }
        ]
    })
    
    
    roles = boto3.client('iam', region_name='us-east-1')
    rolecnt = 0
    while rolecnt < 10:
        rolecnt += 1
        roles.create_role(
            RoleName='RoleName' + str(rolecnt),
            AssumeRolePolicyDocument = assume_role_policy_document,
            Description='RoleName' + str(rolecnt),
            Tags=[
                {
                    'Key': 'mock_role',
                    'Value': 'mock_role' + str(rolecnt)
                },
            ]
        )
        

    """
    Test if our mock role(s) are ready and we can return it
    """
    
    assert iam_scanner_resource.resource_name == 'AWS::IAM::Role'
    
    
    scan_result = iam_scanner_resource.gather(region='us-east-1')

    assert len(scan_result) == rolecnt

    assert  any("RoleName" in s for s in scan_result)
    







import json
import yaml
from pprint import pprint
from cfn_sweeper.artwork import Artwork
class ScanReport():
    def __init__(self):
       self.__report_data = {}

    def add_resource_results(self, resource_type:str, managed_resources:list, unmanaged_resources:list):
        self.__report_data[resource_type] = {
            'managed': managed_resources,
            'unmanaged': unmanaged_resources
        }
    
    def print_to_yaml(self):    
        print(yaml.dump(self.__report_data))
    
    def print_to_json(self):
        print(json.dumps(self.__report_data))
        
    def print_to_pretty(self):
        Artwork.art()
        pprint(self.__report_data)


    def print_to_stdout(self):
        result = []

        for key in self.__report_data.keys():
            result.extend(self.__report_data[key]['unmanaged'])
            

        for unmanaged_item in result:
            print(unmanaged_item)
